# Copyright 2004, @Last Software, Inc.
# The code related to click 3 points was reused here from an original
# example titled rectangle.rb by @Last Software, Inc. 
# To it I added the window code.
# Name:           WindowGlassFrame
# Description:    Create a simple window, with frame, trim & glazing 
# Usage:          click 3 points, on an EXISTING OPENING, creates a window frame with glazing
# Date:           2008 
# Type:           Tool
# Revised:        with thanks to Jim Foltz for the 
#------------------------------------------------------------------------------------------------
require 'sketchup.rb'

class WindowGlassFrame

def initialize
    @ip = Sketchup::InputPoint.new
    @ip1 = Sketchup::InputPoint.new
    reset

    # sets the default Window settings
    @jthick = 1.5.inch if not @jthick
    @fwidth = 6.inch if not @fwidth 
    @gthick = 1.inch if not @gthick
    @ginset = 3.inch if not @ginset
    @etthick = 1.inch if not @etthick
    @etwidth = 5.inch if not @etwidth
    
    # Dialog box
    prompts = ["Frame Thickness", "Frame Width", "Glass Thickness", "Glass Inset","Exterior Trim Thickness ", "Exterior Trim Width"]
    values = [@jthick, @fwidth, @gthick, @ginset, @etthick, @etwidth]
    results = inputbox prompts, values, "Window parameters"

    return if not results
    @jthick, @fwidth, @gthick, @ginset, @etthick, @etwidth = results
end

def reset
    @pts = []
    @state = 0
    @ip1.clear
    @drawn = false
    Sketchup::set_status_text "", SB_VCB_LABEL
    Sketchup::set_status_text "", SB_VCB_VALUE
    Sketchup::set_status_text "Click Top Left to start"
    @shift_down_time = Time.now
end

def activate
    self.reset
end

def deactivate(view)
    view.invalidate if @drawn
end

def set_current_point(x, y, view)
    if( !@ip.pick(view, x, y, @ip1) )
        return false
    end
    need_draw = true
    
    # Set the tooltip that will be displayed
    view.tooltip = @ip.tooltip
        
    # Compute points
    case @state
    when 0
        @pts[0] = @ip.position
        @pts[4] = @pts[0]
        need_draw = @ip.display? || @drawn
    when 1
        @pts[1] = @ip.position
        @length = @pts[0].distance @pts[1]
        Sketchup::set_status_text @length.to_s, SB_VCB_VALUE
    when 2
        pt1 = @ip.position
        pt2 = pt1.project_to_line @pts
        vec = pt1 - pt2

        @pt1 = @ip.position
        @pt2 = pt1.project_to_line @pts
        @vec = @pt1 - @pt2


        @width = vec.length
        if( @width > 0 )
            # test for a square
            square_point = pt2.offset(vec, @length)
            if( view.pick_helper.test_point(square_point, x, y) )
                @width = @length
                @pts[2] = @pts[1].offset(vec, @width)
                @pts[3] = @pts[0].offset(vec, @width)
                view.tooltip = "Square"
            else
                @pts[2] = @pts[1].offset(vec)
                @pts[3] = @pts[0].offset(vec)
            end
        else
            @pts[2] = @pts[1]
            @pts[3] = @pts[0]
        end
        Sketchup::set_status_text @width.to_s, SB_VCB_VALUE
    end

    view.invalidate if need_draw
end

def onMouseMove(flags, x, y, view)
    self.set_current_point(x, y, view)
end

def create_rectangle
#--------------------------------------------------------------------------
    model=Sketchup.active_model
    model.start_operation "Create Window"
    entities=model.active_entities
    
    # creates a new point from p1 in the direction of p2-p3 with length d
    # params are Point3d, 2 vertices, a length, returns a Point3d
    def translate(p1, p2, p3, d)
        v = p3 - p2
        v.length = d
        trans=Geom::Transformation.translation(v)
        return p1.transform(trans)
    end   
          
            
    #------create a new set of points from the original 3 point pick
    @pt0=Geom::Point3d.new(@pts[0][0], @pts[0][1], @pts[0][2])
    @pt1=Geom::Point3d.new(@pts[1][0], @pts[1][1], @pts[1][2])
    @pt2=Geom::Point3d.new(@pts[2][0], @pts[2][1], @pts[2][2])
    @pt3=Geom::Point3d.new(@pts[3][0], @pts[3][1], @pts[3][2])
    
    #------find the points for the head btw. @pt0 & #pt3
    @pt0j=translate(@pt0, @pt0, @pt3, @jthick)
    #------find the points for the head btw. @pt1 & #pt2
    @pt1j=translate(@pt1, @pt1, @pt2, @jthick)
    #------find the points for the sill btw. @pt0 & #pt3
    @pt2j=translate(@pt2, @pt2, @pt1, @jthick)
    #------find the points for the sill btw. @pt1 & #pt2
    @pt3j=translate(@pt3, @pt3, @pt0, @jthick)
    #------find the points for the left jamb btw. @pt0j & #pt1j
    @pt0jj=translate(@pt0j, @pt0j, @pt1j, @jthick)
    #------find the points for the left jamb btw. @pt3j & #pt2j
    @pt3jj=translate(@pt3j, @pt3j, @pt2j, @jthick)
    #------find the points for the right jamb btw. @pt0 & #pt3
    @pt1jj=translate(@pt1j, @pt1j, @pt0j, @jthick)
    #------find the points for the right jamb btw. @pt1 & #pt2
    @pt2jj=translate(@pt2j, @pt2j, @pt3j, @jthick)
            
    #------Draw the Frame 
    group=entities.add_group
    entities=group.entities    
    base=entities.add_face(@pt0, @pt1, @pt2, @pt3)
    base.pushpull @fwidth
    base=entities.add_face(@pt0jj, @pt1jj, @pt2jj, @pt3jj)
    base.pushpull -@fwidth
    
    #------find the points for the head 
    @pt0tt=translate(@pt0, @pt0, @pt1, -@etwidth+@jthick) 
    @pt1tt=translate(@pt1, @pt1, @pt0, -@etwidth+@jthick) 
    #------find the points for the sill 
    @pt3tt=translate(@pt3, @pt3, @pt2, -@etwidth+@jthick)
    @pt2tt=translate(@pt2, @pt2, @pt3, -@etwidth+@jthick)
    #------find the points for the head 
    @pt0t=translate(@pt0tt, @pt0tt, @pt3tt, -@etwidth+@jthick)
    @pt1t=translate(@pt1tt, @pt1tt, @pt2tt, -@etwidth+@jthick)
    #------find the points for the sill 
    @pt3t=translate(@pt3tt, @pt3tt, @pt0tt, -@etwidth+@jthick)
    @pt2t=translate(@pt2tt, @pt2tt, @pt1tt, -@etwidth+@jthick)
    
if( (@etthick != 0) and (@etwidth != 0) ) # thanks to Jim Foltz for this fix
    #------Draw the exterior Trim
    group=entities.add_group
    entities=group.entities
    base=entities.add_face(@pt0t, @pt1t, @pt2t, @pt3t)
    base.pushpull -@etthick
    base=entities.add_face(@pt0jj, @pt1jj, @pt2jj, @pt3jj)
    base.pushpull -@etthick     
end
    #------Do Glazing 
    group=entities.add_group
    entities=group.entities
    base=entities.add_face(@pt0jj, @pt1jj, @pt2jj, @pt3jj)
    base.material=Sketchup::Color.new(163,204,204) #insert your own RGB Color numbers
    base.material.alpha = 0.6 
    base.pushpull -@gthick
    self.reset
    normal=base.normal
    normal.length= -@ginset-@gthick
    tr=Geom::Transformation.translation(normal.reverse)
    group.transform! tr
    
    model.commit_operation
end

def increment_state
    @state += 1
    case @state
    when 1
        @ip1.copy! @ip
        Sketchup::set_status_text "Click Top Right for 2nd point"
        Sketchup::set_status_text "", SB_VCB_LABEL
        Sketchup::set_status_text "", SB_VCB_VALUE
    when 2
        @ip1.clear
        Sketchup::set_status_text "Click Btm Right for 3rd point"
        Sketchup::set_status_text "", SB_VCB_LABEL
        Sketchup::set_status_text "", SB_VCB_VALUE
    when 3
        self.create_rectangle
    end
end

def onLButtonDown(flags, x, y, view)
    self.set_current_point(x, y, view)
    self.increment_state
    view.lock_inference
end

def onCancel(flag, view)
    view.invalidate if @drawn
    self.reset
end

# This is called when the user types a value into the VCB
def onUserText(text, view)
    # The user may type in something that we can't parse as a length
    # so we set up some exception handling to trap that
    begin
        value = text.to_l
    rescue
        # Error parsing the text
        UI.beep
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value
    
    case @state
    when 1
        # update the width
        vec = @pts[1] - @pts[0]
        if( vec.length > 0.0 )
            vec.length = value
            @pts[1] = @pts[0].offset(vec)
            view.invalidate
            self.increment_state
        end
    when 2
        # update the height
        vec = @pts[3] - @pts[0]
        if( vec.length > 0.0 )
            vec.length = value
            @pts[2] = @pts[1].offset(vec)
            @pts[3] = @pts[0].offset(vec)
            self.increment_state
        end
    end
end

def getExtents
    bb = Geom::BoundingBox.new
    case @state
    when 0
        # We are getting the first point
        if( @ip.valid? && @ip.display? )
            bb.add @ip.position
        end
    when 1
        bb.add @pts[0]
        bb.add @pts[1]
    when 2
        bb.add @pts
    end
    bb
end

def draw(view)
    @drawn = false
    
    # Show the current input point
    if( @ip.valid? && @ip.display? )
        @ip.draw(view)
        @drawn = true
    end

    # show the rectangle
    if( @state == 1 )
        # just draw a line from the start to the end point
        view.set_color_from_line(@ip1, @ip)
        inference_locked = view.inference_locked?
        view.line_width = 3 if inference_locked
        view.draw(GL_LINE_STRIP, @pts[0], @pts[1])
        view.line_width = 1 if inference_locked
        @drawn = true
    elsif( @state > 1 )
        # draw the curve
        view.drawing_color = "black"
        view.draw(GL_LINE_STRIP, @pts)
        @drawn = true
    end
end

def onKeyDown(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && rpt == 1 )
        @shift_down_time = Time.now
        
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
            view.lock_inference
        elsif( @state == 0 )
            view.lock_inference @ip
        elsif( @state == 1 )
            view.lock_inference @ip, @ip1
        end
    end
end

def onKeyUp(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end

  
end # of class WindowGlassFrame
#=============================================================================
def WindowGlassFrame
Sketchup.active_model.select_tool WindowGlassFrame.new
end

if( not file_loaded?("WindowGlassFrame.rb") )
    UI.menu("Plugins").add_item("Window Glass Frame") { Sketchup.active_model.select_tool WindowGlassFrame.new }
end
#-----------------------------------------------------------------------------
file_loaded("WindowGlassFrame.rb") # load"WindowGlassFrame.rb"
